import {Component, OnDestroy, OnInit} from "@angular/core";
import {CapabilityRequest} from "../../model/capability-request";
import {ActivatedRoute} from "@angular/router";
import {DataRetrieverService} from "../../services/data-retriever.service";
import {CapabilityExecution} from "../../model/capability-execution";
import {CapabilityRequestService} from "../../services/capability-request.service";
import {CapabilityVersion} from "../../model/capability-version";
import {PollingDataUpdaterService} from "../../services/polling-data-updater.service";
import {AlertService} from "../../../shared/alert/alert.service";
import {JsonObject} from "@angular/compiler-cli/ngcc/src/packages/entry_point";
import {Subject} from "rxjs";
import {takeUntil} from "rxjs/operators";
import {Capability} from "../../model/capability";

@Component({
  selector: "app-capability-request",
  templateUrl: "./capability-request.component.html",
  styleUrls: ["./capability-request.component.scss"],
})
export class CapabilityRequestComponent implements OnInit, OnDestroy {
  public capabilityRequest: CapabilityRequest;
  public capability: Capability;
  public capabilityExecution: CapabilityExecution;
  public currentVersion: CapabilityVersion;

  private ngUnsubscribe = new Subject(); // to signal when all subscribers should unsubscribe (triggered in tear down)

  // Observer for capability request objects
  private capabilityRequestObserver = {
    next: (request) => {
      this.capabilityRequest = request;
      this.capabilityExecution = request.current_execution;
      this.dataRetriever
        .getCapability(this.capabilityRequest.capability_name)
        .pipe(takeUntil(this.ngUnsubscribe))
        .subscribe(this.capabilityObserver);
    },
    error: (error) => console.error("Error when retrieving capability request:" + error),
  };
  // Observer for capability objects
  private capabilityObserver = {
    next: (capability) => {
      this.capability = capability;
    },
    error: (error) => console.error("Error when retrieving capability:" + error),
  };
  // Observer for capability version objects
  private capabilityVersionObserver = {
    next: (capabilityVersion) => {
      if (capabilityVersion !== undefined) {
        this.currentVersion = capabilityVersion;
      } else {
        console.error("Current version returned undefined.");
      }
    },
    error: (error) => console.error("Error when retrieving current version:" + error),
  };

  constructor(
    private route: ActivatedRoute,
    private dataRetriever: DataRetrieverService,
    private capabilityRequestService: CapabilityRequestService,
    private pollingDataUpdaterService: PollingDataUpdaterService,
    private alertService: AlertService,
  ) {
    const requestID = parseInt(this.route.snapshot.paramMap.get("id"));
    this.pollingDataUpdaterService
      .getDataPoller$(this.capabilityRequestService.getCapabilityRequestURL(requestID))
      .pipe(takeUntil(this.ngUnsubscribe))
      .subscribe(this.capabilityRequestObserver);
    this.pollingDataUpdaterService
      .getDataPoller$(this.capabilityRequestService.getLatestVersionURL(requestID))
      .pipe(takeUntil(this.ngUnsubscribe))
      .subscribe(this.capabilityVersionObserver);
  }

  alertMessage(eventDetails: JsonObject): void {
    let message = "";
    let title = "";

    switch (eventDetails.type) {
      case "carta": {
        title = "Launching CARTA";
        message =
          "An email has been sent to <strong>" +
          eventDetails.email +
          "</strong> with a link to your CARTA instance.";
        this.alertService.info(message, title);
        break;
      }
      case "error": {
        message = "There has been an error with your request";
        title = "Error";
        this.alertService.error(message, title);
        break;
      }
    }
  }

  /**
   * Pass this request's QA
   *
   * @param requestId the ID of the request on which to pass QA
   */
  public passQa(requestId: string) {
    const addRestCallObservable = {
      next: (response) => {
        console.log(">>> GOT RESPONSE:'" + response + "'");
      },
      error: (error) => console.error("Error when trying to pass QA on request " + requestId + ": " + error),
    };

    return this.capabilityRequestService.sendQaPassRequest(parseInt(requestId)).subscribe(addRestCallObservable)

  }
  ngOnInit(): void {}

  ngOnDestroy(): void {
    // Prevent memory leakage by unsubscribing all observers
    this.ngUnsubscribe.next();
    this.ngUnsubscribe.complete();
  }
}
