""" Tests of CapabilityQueue """

from typing import List

# pylint: disable=E0401

from testing.utils.conftest import mock_workflow_service, mock_capability_info
from workspaces.capability.schema import CapabilityExecution
from workspaces.capability.schema_interfaces import CapabilityExecutionIF
from workspaces.capability.services.interfaces import (
    QueueReporterIF,
    ExecutionManagerIF,
    QueueReport,
)
from workspaces.capability.services.queue_manager import CapabilityQueueManager
from workspaces.system.schema import AbstractFile

pytest_plugins = ["testing.utils.conftest"]


class FakeQueueReporterExecutionManager(QueueReporterIF, ExecutionManagerIF):
    """
    A testing mock that pretends to be both the queue reporter and execution manager.
    Our notions of both are pretty limited, basically moving things from one list to
    another so that we can get proper reports, but otherwise not doing anything real.
    """

    submitted = []
    executing = []
    completed = []
    allowed = 3
    workflow_service = mock_workflow_service
    capability_info = mock_capability_info

    def report(self) -> List[QueueReport]:
        return [QueueReport("fake", len(self.submitted), len(self.executing), self.allowed)]

    def waiting_executions(self, queue: str, count: int) -> List[CapabilityExecutionIF]:
        return self.submitted[:count]

    def submit_workflow_request(
        self,
        execution_id: int,
        workflow_name: str,
        workflow_args: dict,
        files: List[AbstractFile],
    ):
        execution = mock_capability_info.lookup_execution(execution_id)
        self.submitted.append(execution)

    def submit(self, execution: CapabilityExecutionIF):
        self.submitted.append(execution)

    def execute(self, execution: CapabilityExecutionIF):
        self.submitted.remove(execution)
        self.executing.append(execution)

    def complete(self, execution: CapabilityExecutionIF):
        print(f">>> currently executing: {[exec.id for exec in self.executing]}")
        self.executing.remove(execution)
        self.completed.append(execution)


def test_capability_queue_manager():
    # let's start by putting five things into the execution manager
    test_reporter_executor = FakeQueueReporterExecutionManager()
    executions = []
    for i in range(1, 6):
        execution = CapabilityExecution(id=i)
        test_reporter_executor.submit(execution)
        executions.append(execution)

    for i in range(1, 3):
        execution = test_reporter_executor.submitted[i]
        test_reporter_executor.execute(execution)
        executions.append(execution)

    # now let's make our queue manager and see what the report tells us
    queue = CapabilityQueueManager(test_reporter_executor, test_reporter_executor)
    queue.launch_if_availability()

    # we should now have 3 executing and 2 submitted
    assert test_reporter_executor.report()[0].executing == 3
    assert test_reporter_executor.report()[0].waiting == 2

    # now let's finish a couple of them
    test_reporter_executor.complete(executions[1])
    queue.execution_complete()
    test_reporter_executor.complete(executions[3])
    queue.execution_complete()

    # we should now have 3 executing and 0 submitted
    assert test_reporter_executor.report()[0].executing == 3
    assert test_reporter_executor.report()[0].waiting == 0

    # let's complete a couple more, this time no notification in between
    test_reporter_executor.complete(executions[2])
    test_reporter_executor.complete(executions[4])
    queue.execution_complete()

    # we should now have 1 executing and 0 submitted
    assert test_reporter_executor.report()[0].executing == 1
    assert test_reporter_executor.report()[0].waiting == 0

    # let's add four more
    for i in range(6, 10):
        execution = CapabilityExecution(id=i)
        test_reporter_executor.submit(execution)
        executions.append(execution)

    queue.execution_requested()

    # now check again and we should have 3 executing and 2 submitted again
    assert test_reporter_executor.report()[0].executing == 3
    assert test_reporter_executor.report()[0].waiting == 2
