""" Creates AMQP message templates """

import logging

# pylint: disable=E0401

from immutable_views import DictView
from workspaces.shared_interfaces import MessageArchitectIF
from workspaces.workflow.schema_interfaces import WorkflowRequestIF

logger = logging.getLogger(__name__)

"""
Class which create an immutable dictionary of amqp message templates for easy access
"""
service_templates = DictView(
    {
        "ws": {
            "service": None,
            "routing_key": None,
            "subject": None,
        },
        "archive": {
            "service": "archive",
            "routing_key": None,
            "application": "workflow",
            "subject": None,
        },
    }
)


workflow_msg_templates = DictView(
    {
        "workflow_failed": {
            "type": "workflow-failed",
        },
        "delivery": {
            "type": "delivery",
            "delivery": None,
        },
        "ingestion_complete": {
            "type": "ingestion-complete",
        },
        "qa_ready": {
            "type": "qa_ready",
        },
        "qa_pass": {
            "type": "qa-pass",
        },
        "qa_fail": {
            "type": "qa-fail",
        },
        "carta_ready": {
            "carta_url": None,
            "type": "carta-ready",
        },
    }
)

archive_msg_templates = DictView(
    {
        "carta_ready": {
            "carta_url": None,
            "message": "carta ready",
        },
        "carta_failed": {
            "carta_url": None,
            "message": "carta failed",
        },
        "seci_failed": {
            "message": "seci failed",
            "status": "failed",
        },
        "seci_complete": {
            "message": "seci complete",
            "status": "complete",
        },
    }
)


class WorkflowMessageArchitect(MessageArchitectIF):
    """Builds workflow AMQP messages"""

    def __init__(
        self,
        request: WorkflowRequestIF = None,
        previous_info=None,
        delivery_info=None,
        carta_url=None,
    ):
        self.subject = request or previous_info
        self.delivery = delivery_info
        self.carta_url = carta_url

    @staticmethod
    def get_message_template(msg_type: str) -> dict:
        return {**service_templates["ws"], **workflow_msg_templates[msg_type]}

    def compose_message(self, msg_type: str) -> DictView:
        if any(item in msg_type for item in ["qa_, carta"]):
            service = "capability"
        else:
            service = "workflow"

        template = self.get_message_template(msg_type)
        template = template.copy()

        template["service"] = service
        template["routing_key"] = service
        template["subject"] = self.subject

        if self.delivery is not None:
            template["delivery"] = self.delivery

        if self.carta_url is not None:
            template["carta_url"] = self.carta_url

        return DictView(template)


class ArchiveMessageArchitect(MessageArchitectIF):
    """Builds archive system AMQP messages"""

    def __init__(self, routing_key: str, request: WorkflowRequestIF, carta_url: str = None):
        self.key = routing_key
        self.subject = request
        self.carta_url = carta_url

    @staticmethod
    def get_message_template(msg_type: str) -> dict:
        return {**service_templates["archive"], **archive_msg_templates[msg_type]}

    def compose_message(self, msg_type: str) -> DictView:
        template = self.get_message_template(msg_type)
        template = template.copy()
        template["subject"] = self.subject
        template["routing_key"] = self.key

        if self.carta_url is not None:
            template["carta_url"] = self.carta_url

        return DictView(template)
